==> bqpackage-1.0.62-py3-none-any.whl: bqpackage/selenium_base/selenium_actions.py <==
"""
This module wrapping basic selenium actions
"""
import re

import allure
from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

MAX_PAGES = 50
MAX_CLICKS = 100
CLICK_INTERVAL = 0.3


@allure.step
class SeleniumActions:

    @allure.step('Selenium actions initialization')
    def __init__(self, driver):
        self._list_of_elements = []
        self._web_element_type = "webelement"
        self._temp_element = None
        self._wait_seconds = 0
        self.driver = driver

    @allure.step
    def pagination(self, container: str, next_btn: str):
        """
        Loop pages until reaches the end

        :arg container : Container
        :arg next_btn : Next button element
        :returns True->successfully paginated, False->error
        """
        for i in self.pagination_iterator(container=container, next_btn=next_btn):
            pass
        return True

    @allure.step
    def pagination_iterator(self, container: str, next_btn: str):
        """
        Loop pages, yield every page for future consume

        :arg container : Container
        :arg next_btn : Next button element
        :returns True->next page succeeded, False->error
        """
        visited = set()
        container_text = self.get_text(element=container, strip=True)
        i = 0
        yield container_text
        while container_text not in visited:
            visited.add(container_text)
            self.click(element=f'{container}{next_btn}')
            container_text = self.get_text(element=container, strip=True)
            i += 1
            assert i < MAX_PAGES, 'looping pages reached to maximum'
            yield container_text

        # exhausted
        return container_text

    @allure.step
    def text_in_pagination(self, container: str, next_btn: str, text: str) -> str:
        """
        Loop pages until reaches the end/criteria met (text in page)

        :arg container : Container
        :arg next_btn : Next button element
        :arg text : Text to look for
        :returns True->Text found on current page, False->error
        """
        text = re.sub(r"\s+", "", text)
        for container_text in self.pagination_iterator(container=container, next_btn=next_btn):
            if text in container_text:
                return text

        assert False, 'failed to find text in pages'

    @allure.step
    def wait_for_text(self, location=None, text=None):
        """
        wait for a text to be presented inside/without element (currently searching by xpath!)
        driver max waiting time is 30 sec.
        :arg location : element to search inside
        :arg text : text to look for
        :returns True->Text found on current element/non element, False->error
        """
        try:
            if location:
                WebDriverWait(self.driver, 30).until(
                    EC.text_to_be_present_in_element((By.CSS_SELECTOR, location), text))
            else:
                text = self.find_element_by_xpath(f"//*[text()='{text}']").text
            return True
        except Exception:
            assert False, f"Failed to find text: {text}"

    @allure.step
    def get_list_of_elements_by_type(self, element_type, element):
        """
        - args -
              element_type: 1 = id, 2 - xpath, 3 - name, 4 - css, 5 - link text"""
        try:
            if element_type == 1:
                self._list_of_elements = self.driver.find_elements(By.ID, element)
            elif element_type == 2:
                self._list_of_elements = self.driver.find_elements(By.XPATH, element)
            elif element_type == 3:
                self._list_of_elements = self.driver.find_elements(By.NAME, element)
            elif element_type == 4:
                self._list_of_elements = self.driver.find_elements(By.CSS_SELECTOR, element)
            elif element_type == 5:
                self._list_of_elements = self.driver.find_elements(By.LINK_TEXT, element)

            return self._list_of_elements

        except Exception as e:
            return

    @allure.step
    def click(self, element, wait=0):
        """
        click on element, optional waiting time.
        :arg element : element to be clicked
        :arg wait : time to wait before clicking
        :returns True->Clicked False->error
        """
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.click()
                return True
        except Exception as e:
            assert False, f"Failed to click on element: {e}"

    @allure.step
    def vclick(self, element, proof: str):
        """
        loop of clicking an element until a proof (text exist on page).
        :arg element : element to be clicked
        :arg proof : text to check after click
        :returns True->Clicked and proof exist False->error
        """
        i = 0
        while proof not in self.driver.page_source:
            try:
                self._temp_element = self.check_all_options(element)
                self._temp_element.click()
                sleep(CLICK_INTERVAL)
                i += 1
                assert i < MAX_CLICKS, 'maximum clicks have been reached'
            except Exception as e:
                pass
        assert proof in self.driver.page_source, f"Failed to click on element: {element}"

    @allure.step
    def hover(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                hover = ActionChains(self.driver).move_to_element(self._temp_element)
                hover.perform()
                return True
        except Exception as e:
            assert False, f"Failed to hover on element ,reason: {e}"

    @allure.step
    def send_keys(self, element=None, text=None, wait=0):
        text = str(text)
        sleep(wait)
        if not element:
            ActionChains(self.driver).send_keys(text).perform()
            return True
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.send_keys(text)
                return True
        except Exception as e:
            assert False, f"Failed to send text to element: {e}"

    @allure.step
    def get_text(self, element, wait=0, strip=False):
        sleep(wait)
        self._temp_element = element
        if self._web_element_type not in str(type(self._temp_element)):
            self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                allure.attach(self._temp_element.text, 'Text', allure.attachment_type.TEXT)
                if strip:
                    return re.sub(r"\s+", "", self._temp_element.text)
                return self._temp_element.text
        except Exception as e:
            assert False, f"Failed to get text to element: {element}"

    @allure.step
    def check_all_options(self, element):
        self._temp_element = self.find_element_by_xpath(element)
        if self._temp_element is not None:
            return self._temp_element

        # check id option
        self._temp_element = self.find_element_by_id(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_css(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_link_text(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_name(element)
        if self._temp_element is not None:
            return self._temp_element

        return

    @allure.step
    def find_element_by_id(self, element):
        try:
            self._temp_element = self.driver.find_element_by_id(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_class_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_class_name(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_xpath(self, element):
        try:
            self._temp_element = self.driver.find_element_by_xpath(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_name(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_css(self, element):
        try:
            self._temp_element = self.driver.find_element_by_css(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_link_text(self, element):
        try:
            self._temp_element = self.driver.find_element_by_link_text(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def get_page_source(self):
        try:
            source = self.driver.page_source
            allure.attach(source, 'Page source', allure.attachment_type.TEXT)
            return source
        except Exception:
            return False

    @allure.step
    def get_body(self):
        try:
            allure.attach(self.driver.find_element_by_tag_name("body").text, 'Body',
                          allure.attachment_type.TEXT)
            return self.driver.find_element_by_tag_name("body").text
        except Exception:
            return False

    @allure.step
    def get_href(self, element):
        try:
            if element is not None:
                self._temp_element = element.get_attribute("href")
                allure.attach(self._temp_element, 'href', allure.attachment_type.TEXT)
                return self._temp_element

        except Exception:
            return False

==> bqpackage-1.0.710-py3-none-any.whl: bqpackage/selenium_base/selenium_actions.py <==
"""
This module wrapping basic selenium actions
"""
import re

import allure
from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

MAX_PAGES = 50
MAX_CLICKS = 30
MAX_NOT_EXISTS = 2
GLOBAL_MAX = 1000
CLICK_INTERVAL = 0.3


@allure.step
class SeleniumActions:

    @allure.step('Selenium actions initialization')
    def __init__(self, driver):
        self._list_of_elements = []
        self._web_element_type = "webelement"
        self._temp_element = None
        self._wait_seconds = 0
        self.driver = driver

    @allure.step
    def pagination(self, container: str, next_btn: str):
        """
        Loop pages until reaches the end

        :arg container : Container
        :arg next_btn : Next button element
        :returns True->successfully paginated, False->error
        """
        for i in self.pagination_iterator(container=container, next_btn=next_btn):
            pass
        return True

    @allure.step
    def pagination_iterator(self, container: str, next_btn: str):
        """
        Loop pages, yield every page for future consume

        :arg container : Container
        :arg next_btn : Next button element
        :returns page text->when there's text, False->error
        """
        visited = set()
        container_text = self.get_text(element=container, strip=True)
        i = 0
        yield container_text
        while container_text not in visited:
            visited.add(container_text)
            self.click(element=f'{container}{next_btn}')
            container_text = self.get_text(element=container, strip=True)
            i += 1
            if not i < MAX_PAGES:
                raise RuntimeError(f'looping pages reached maximum {i}/{MAX_PAGES}')
            yield container_text

        # exhausted
        return container_text

    @allure.step
    def text_in_pagination(self, container: str, next_btn: str, text: str):
        """
        Loop pages until reaches the end/criteria met (text in page)

        :arg container : Container
        :arg next_btn : Next button element
        :arg text : Text to look for
        :returns Text->Text found on current page, False->error
        """
        text = re.sub(r"\s+", "", text)
        for container_text in self.pagination_iterator(container=container, next_btn=next_btn):
            if text in container_text:
                return text

        return False

    @allure.step
    def elements_iterator(self, element: str):
        """
        Loop elements from the same kind, yield every element for future consume => (element)[i]

        For ex. if your element type is "//div[contains(text(), 'blabla')]" and there plenty of it
        You will pass this element and the iterator wil yield you a wrap of
        (//div[contains(text(), 'blabla')])[i] each time

        :arg element : kind of element
        :returns WebElement->if found, False->element not found
        """
        i = 0
        el = self.check_all_options(element=f'({element})[{i}]')
        while el:
            yield el
            i += 1
            if not i < GLOBAL_MAX:
                raise RuntimeError(f'looping elements reached maximum {i}/{GLOBAL_MAX}')
            el = self.check_all_options(element=f'({element})[{i}]')

        # exhausted
        return el

    @allure.step
    def wait_for_text(self, location=None, text=None):
        """
        wait for a text to be presented inside/without element (currently searching by xpath!)
        driver max waiting time is 30 sec.
        :arg location : element to search inside
        :arg text : text to look for
        :returns True->Text found on current element/non element, False->error
        """
        try:
            if location:
                WebDriverWait(self.driver, 30).until(
                    EC.text_to_be_present_in_element((By.CSS_SELECTOR, location), text))
            else:
                text = self.find_element_by_xpath(f"//*[text()='{text}']").text
            return True
        except Exception:
            raise RuntimeError(f'failed to find {text} in page')

    @allure.step
    def get_list_of_elements_by_type(self, element_type, element):
        """
        - args -
              element_type: 1 = id, 2 - xpath, 3 - name, 4 - css, 5 - link text"""
        try:
            if element_type == 1:
                self._list_of_elements = self.driver.find_elements(By.ID, element)
            elif element_type == 2:
                self._list_of_elements = self.driver.find_elements(By.XPATH, element)
            elif element_type == 3:
                self._list_of_elements = self.driver.find_elements(By.NAME, element)
            elif element_type == 4:
                self._list_of_elements = self.driver.find_elements(By.CSS_SELECTOR, element)
            elif element_type == 5:
                self._list_of_elements = self.driver.find_elements(By.LINK_TEXT, element)

            return self._list_of_elements

        except Exception as e:
            raise RuntimeError(f'failed to get {element}')

    @allure.step
    def click(self, element, wait=0):
        """
        click on element, optional waiting time.
        :arg element : element to be clicked
        :arg wait : time to wait before clicking
        :returns True->Clicked False->error
        """
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.click()
                return True
        except Exception as e:
            raise RuntimeError(f'failed to click {element}')

    @allure.step
    def vclick(self, element, proof: str):
        """
        loop of clicking an element until a proof (text exist on page).
        :arg element : element to be clicked
        :arg proof : text to check after click
        :returns True->Clicked and proof exist False->error
        """
        clicks = 0
        not_exists = 0
        while proof not in self.driver.page_source:
            self._temp_element = self.check_all_options(element)

            if not self._temp_element:
                not_exists += 1
                if not_exists > MAX_NOT_EXISTS:
                    raise RuntimeError(f'object exceeded maximum loading tries {not_exists}/{MAX_NOT_EXISTS}')

            else:
                try:
                    self._temp_element.click()
                except Exception as e:
                    pass

                sleep(CLICK_INTERVAL)
                clicks += 1
                if clicks > MAX_CLICKS:
                    raise RuntimeError(f'maximum clicks passed limit {clicks}/{MAX_CLICKS}')

        if not proof in self.driver.page_source:
            raise RuntimeError(f"failed to click on element: {element}")

    @allure.step
    def hover(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                hover = ActionChains(self.driver).move_to_element(self._temp_element)
                hover.perform()
                return True
        except Exception as e:
            raise RuntimeError(f'failed to hover {element}')

    @allure.step
    def send_keys(self, element=None, text=None, wait=0):
        text = str(text)
        sleep(wait)
        if not element:
            ActionChains(self.driver).send_keys(text).perform()
            return True
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.send_keys(text)
                return True
        except Exception as e:
            raise RuntimeError(f'failed to send keys {element}')

    @allure.step
    def get_text(self, element, wait=0, strip=False):
        sleep(wait)
        self._temp_element = element
        if self._web_element_type not in str(type(self._temp_element)):
            self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                allure.attach(self._temp_element.text, 'Text', allure.attachment_type.TEXT)
                if strip:
                    return re.sub(r"\s+", "", self._temp_element.text)
                return self._temp_element.text
        except Exception as e:
            raise RuntimeError(f'failed to get text {element}')

    @allure.step
    def check_all_options(self, element):
        self._temp_element = self.find_element_by_xpath(element)
        if self._temp_element is not None:
            return self._temp_element

        # check id option
        self._temp_element = self.find_element_by_id(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_css(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_link_text(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_name(element)
        if self._temp_element is not None:
            return self._temp_element

        return

    @allure.step
    def find_element_by_id(self, element):
        try:
            self._temp_element = self.driver.find_element_by_id(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_class_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_class_name(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_xpath(self, element):
        try:
            self._temp_element = self.driver.find_element_by_xpath(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_name(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_css(self, element):
        try:
            self._temp_element = self.driver.find_element_by_css(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def find_element_by_link_text(self, element):
        try:
            self._temp_element = self.driver.find_element_by_link_text(element)
            return self._temp_element
        except Exception:
            return False

    @allure.step
    def get_page_source(self):
        try:
            source = self.driver.page_source
            allure.attach(source, 'Page source', allure.attachment_type.TEXT)
            return source
        except Exception:
            return False

    @allure.step
    def get_body(self):
        try:
            allure.attach(self.driver.find_element_by_tag_name("body").text, 'Body',
                          allure.attachment_type.TEXT)
            return self.driver.find_element_by_tag_name("body").text
        except Exception:
            return False

    @allure.step
    def get_href(self, element):
        try:
            if element is not None:
                self._temp_element = element.get_attribute("href")
                allure.attach(self._temp_element, 'href', allure.attachment_type.TEXT)
                return self._temp_element

        except Exception:
            return False
